package org.xbib.interlibrary.slnp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class SlnpClient {

    private static final Logger logger = Logger.getLogger(SlnpClient.class.getName());

    public SlnpClient() {
    }

    public List<String> send(String host, int port, String cmd)
            throws IOException {
        List<String> response = new ArrayList<>();
        InetAddress inetAddress = InetAddress.getByName(host);
        SocketAddress socketAddress = new InetSocketAddress(inetAddress, port);
        Socket socket = new Socket();
        try (socket) {
            int connectTimeout = 5 * 1000;
            socket.connect(socketAddress, connectTimeout);
            int readTimeout = 5 * 1000;
            socket.setSoTimeout(readTimeout);
            logger.log(Level.INFO, "client connected to " + socket.getInetAddress() + ":" + socket.getPort());
            InputStream inputStream = socket.getInputStream();
            OutputStream outputStream = socket.getOutputStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, Charset.defaultCharset()));
            String banner = bufferedReader.readLine();
            outputStream.write((cmd + "\nSLNPQuit\n").getBytes(StandardCharsets.UTF_8));
            outputStream.flush();
            logger.log(Level.INFO, "request = " + cmd + "\nSLNPQuit\n" );
            int attempts = 0;
            while (inputStream.available() == 0 && attempts < 50) {
                attempts++;
                Thread.sleep(200);
            }
            while (inputStream.available() > 0) {
                String s = bufferedReader.readLine();
                while (s != null && !s.isEmpty() && !s.contains("SLNPEndOfData")) {
                    response.add(s);
                    s = bufferedReader.readLine();
                }
            }
            logger.log(Level.INFO, "response = " + response);
        } catch (Throwable t) {
            logger.log(Level.SEVERE, "SLNPCLient(" + host + ":" + port + "): " + t.getMessage(), t);
        }
        return response;
    }

    public List<String> send(String host, int port, int connectTimeout, int readTimeout, Charset charset, String cmd)
            throws IOException {
        List<String> response = new ArrayList<>();
        InetAddress inetAddress = InetAddress.getByName(host);
        SocketAddress socketAddress = new InetSocketAddress(inetAddress, port);
        Socket socket = new Socket();
        try (socket) {
            socket.connect(socketAddress, connectTimeout);
            socket.setSoTimeout(readTimeout);
            InputStream inputStream = socket.getInputStream();
            OutputStream outputStream = socket.getOutputStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, charset));
            String banner = bufferedReader.readLine();
            outputStream.write((cmd + "\nSLNPQuit\n").getBytes(StandardCharsets.UTF_8));
            outputStream.flush();
            while (true) {
                String s = bufferedReader.readLine();
                if (s != null && !s.isEmpty() && !s.startsWith("SLNPEndOfData")) {
                    response.add(s);
                } else {
                    break;
                }
            }
        }
        return response;
    }
}
